﻿##########################################################################
#独立な1要因の実験データを分析
#■入力
#y: ベクトル形式の特性値
#A: ベクトル形式の水準(1からaまで、抜けなしで、整数で指定)
#prior=F : 論理値。Tなら事前分布の範囲を指定。Fなら指定せず。
#mL=-1000, mH=1000, sL=0, sH=100 : 事前分布のパラメタ
#prob=c(0.025, 0.05, 0.5, 0.95, 0.975): 事後分布で報告する確率点
#see=1234,cha=5,war=1000,ite=21000 : MCMC関連のパラメタ
#■出力
# fit:stanの出力, par:母数リスト, prob:確率ベクトル, 
# muA:水準の平均, sigmaE:水準内sd, sigmaA:水準間sd, eta2:説明率
# delta:効果量, mu:全平均, aj;水準の効果, Ubig:水準の効果0以上,
# Usma:水準の効果0以下が真なら1偽なら0, log_lik:対数尤度
##########################################################################
E1Ind <- function(y,A,prior=F,mL=-1000, mH=1000, sL=0, sH=100,
         prob=c(0.025, 0.05, 0.5, 0.95, 0.975),
         see=1234, cha=5, war=1000, ite=21000)
{
   library(cmdstanr)              #パッケージcmdstanrの呼び出し
   library(posterior)             #パッケージposteriorの呼び出し
   if (prior) {
     dat <- list(n=length(y),a=max(A),y=y,A=A,mL=mL,mH=mH,sL=sL,sH=sH)
     scr <- "stan/E1IndPT.stan"                           # Stan ファイル名
   } else {
     dat <- list(n=length(y),a=max(A),y=y,A=A)
     scr <- "stan/E1IndPF.stan"                           # Stan ファイル名
   }
   par<-c("muA","sigmaE","sigmaA","eta2","delta",
       "mu","aj","Ubig","Usma","U2","log_lik")  # サンプリング変数
      mod <- cmdstan_model(scr)                   #コンパイル
      csrfit <- mod$sample(data=dat,chains=cha,iter_sampling=ite,
         iter_warmup=war,parallel_chains=cha,seed=see)    #MCMC
      draw<-csrfit$draws(par)
      ext<-as_draws_df(draw)
      (colnames(ext) <- gsub("\\[|\\]|,", "", colnames(ext)))
      a<-max(A)
   muA<-as.matrix(ext[,1:a]);  sigmaE<-as.matrix(ext$sigmaE);
   sigmaA<-as.matrix(ext$sigmaA); eta2<-as.matrix(ext$eta2);
   delta<-as.matrix(ext$delta); mu<-as.matrix(ext$mu); 
   aj<-as.matrix(ext[,c((a+6):(2*a+5))]); 
   Ubig<-as.matrix(ext[,c((2*a+6):(3*a+5))]); 
   Usma<-as.matrix(ext[,c((3*a+6):(4*a+5))]);
   log_lik<-as.matrix(ext$log_lik);
   out<-list(prior=prior,ext=ext,draw=draw,par=par,prob=prob,muA=muA,
      sigmaE=sigmaE,sigmaA=sigmaA,eta2=eta2,delta=delta,mu=mu,aj=aj,
      Ubig=Ubig,Usma=Usma,log_lik=log_lik)
   class(out)<-'E1Ind'
   return(invisible(out))
}
##########################################################################
#２つの水準の比較（優越率は出力せず、閾上率の基準点はベクトルで指定）
#■入力
#x:クラス 'E1Ind'のオブジェクト
#degits=3 : 小数の丸め
#I: 整数、平均値の大きい水準
#J: 整数、平均値の小さい水準
#cr1: 閾上率の基準点をベクトルで指定
##########################################################################
E1between<-function(x,degits=3,I,J,cr1=0)
{
 G<-matrix(0,length(x$mu),3)
   G[,1] <- x$muA[,I]-x$muA[,J];
   G[,2] <- G[,1]/x$sigmaE;
   G[,3] <- pnorm(x$muA[,I],x$muA[,J],x$sigmaE);
   lab<-c("平均値差","デルタ","非重複度")
 H<-matrix(0,length(x$mu),length(cr1))
   for (i in 1:length(cr1)){
      H[,i] <- pnorm(((G[,1]-cr1[i])/(sqrt(2)*x$sigmaE)), 0.0, 1.0);
      lab<-c(lab,paste("閾上率(",cr1[i],")",sep=""));}
 Gc<-matrix(0,3+length(cr1),7)
   for (i in 1:3)          {Gc[i,]<-gqcal(G[,i])}
   for (i in 1:length(cr1)){Gc[i+3,]<-gqcal(H[,i])}
   colnames(Gc)<-c("EAP","post.sd","2.5%","5%","50%","95%","97.5%")
   rownames(Gc)<-lab
   round(Gc,degits)
}
