﻿##########################################################################
#独立な2要因の実験データを分析
#■入力
#y: ベクトル形式の特性値
#A: ベクトル形式の水準(1からaまで、抜けなしで、整数で指定)
#B: ベクトル形式の水準(1からbまで、抜けなしで、整数で指定)
#prior=F : 論理値。Tなら事前分布の範囲を指定。Fなら指定せず。
#mL=-1000, mH=1000, sL=0, sH=100 : 事前分布のパラメタ
#prob=c(0.025, 0.05, 0.5, 0.95, 0.975): 事後分布で報告する確率点
#see=1234,cha=5,war=1000,ite=21000 : MCMC関連のパラメタ
#■出力
# fit:stanの出力, par:母数リスト, prob:確率ベクトル, 
#mu;全平均, [a]muA;A平均, [b]muB;B平均, [a,b]muAB;交互作用, 
#[a,b]cellmean;セル平均値,sigmaA;A標準偏差, sigmaB;B標準偏差,sigmaAB;AB標準偏差
#sigmaE;E標準偏差,eta2A;A説明率,eta2B;B説明率,eta2AB;AB説明率,eta2T;全説明率
#deltaA;A効果量, deltaB;B効果量, deltaAB;AB効果量, UbigA[a];A論理値0以上確率, 
#UsmaA[a];A論理値0以下確率, UbigB[b];B論理値0以上確率,UsmaB[b];B論理値0以下確率
#UbigAB[a,b];AB論理値0以上確率, UsmaAB[a,b];AB論理値0以下確率, 
#U2A[a,a];Aの2水準比較, U2B[b,b];Bの2水準比較, log_lik;対数尤度
##########################################################################
E2Ind <- function(y,A,B,prior=F,mL=-1000, mH=1000, sL=0, sH=100,
prob=c(0.025, 0.05, 0.5, 0.95, 0.975),
see=1234, cha=5, war=1000, ite=21000)
{
   library(cmdstanr)              #パッケージcmdstanrの呼び出し
   library(posterior)             #パッケージposteriorの呼び出し
   a<-length(unique(A)); b<-length(unique(B));
   if (prior) {
     dat <- list(n=length(y),a=a,b=b,y=y,A=A,B=B,mL=mL,mH=mH,sL=sL,sH=sH)
     scr <- "stan/E2IndPT.stan"                           # Stan ファイル名
   } else {
     dat <- list(n=length(y),a=a,b=b,y=y,A=A,B=B)
     scr <- "stan/E2IndPF.stan"                           # Stan ファイル名
   }
   par<-c("mu","muA","muB","muAB","cellmean","sigmaA","sigmaB","sigmaAB",
       "sigmaE","eta2A","eta2B","eta2AB","eta2T","deltaA","deltaB","deltaAB",
       "UbigA","UsmaA","UbigB","UsmaB","UbigAB","UsmaAB","U2A","U2B","log_lik")
      mod <- cmdstan_model(scr)                   #コンパイル
      csrfit <- mod$sample(data=dat,chains=cha,iter_sampling=ite,
         iter_warmup=war,parallel_chains=cha,seed=see)    #MCMC
      draw<-csrfit$draws(par)
      ext<-as_draws_df(draw)
      colnames(ext) <- gsub("\\[|\\]|,", "", colnames(ext))
   out<-list(prior=prior,ext=ext,draw=draw,par=par,prob=prob,
      mu=as.matrix(ext$mu),muA=as.matrix(ext[,c(2:(a+1))]),
      muB=as.matrix(ext[,c((a+2):(a+b+1))]),
      muAB=as.matrix(ext[,c((a+b+2):(a+b+a*b+1))]),
      cellmean=as.matrix(ext[,c((a+b+a*b+2):(a+b+2*a*b+1))]),
      sigmaA=as.matrix(ext$sigmaA), sigmaB=as.matrix(ext$sigmaB),
      sigmaAB=as.matrix(ext$sigmaAB), sigmaE=as.matrix(ext$sigmaE),
      eta2A=as.matrix(ext$eta2A), eta2B=as.matrix(ext$eta2B),
      eta2AB=as.matrix(ext$eta2AB) ,eta2T=as.matrix(ext$eta2T),
      log_lik=ext$log_lik)
   class(out)<-'E2Ind'
   return(invisible(out))
}
##########################################################################
#２つのセルの比較（優越率は出力せず、閾上率の基準点はベクトルで指定）
#x:クラス 'E2Ind'$cellmeanのオブジェクト
#I=1: 整数、平均値の大きいセル
#J=2: 整数、平均値の小さいセル
#cr1: 閾上率の基準点をベクトルで指定
#degits=3 : 小数の丸め
##########################################################################
E2between<-function(x,I=2,J=3,cr1=0,degits=3)
{
   big<-x$cellmean[,I]; sma<-x$cellmean[,J];
   G<-matrix(0,length(x$mu),3)
   G[,1] <- big-sma;
   G[,2] <- G[,1]/x$sigmaE;
   G[,3] <- pnorm(big,sma,x$sigmaE);
   lab<-c("平均値の差","デルタ","非重複度")
   K<-matrix(0,length(x$mu),length(cr1))
     for (i in 1:length(cr1)){
        K[,i] <- pnorm(((G[,1]-cr1[i])/(sqrt(2)*x$sigmaE)), 0.0, 1.0);
        lab<-c(lab,paste("閾上率(",cr1[i],")",sep=""));}
   Gc<-matrix(0,3+length(cr1),7)
     for (i in 1:3)          {Gc[i,]<-gqcal(G[,i])}
     for (i in 1:length(cr1)){Gc[i+3,]<-gqcal(K[,i])}
     colnames(Gc)<-c("EAP","post.sd","2.5%","5%","50%","95%","97.5%")
     rownames(Gc)<-lab
     cat( I,"番目のセルと、",J,"番目のセルを比較する\n")
     round(Gc,degits)
}
